@@ -2,7 +2,7 @@
 use Date::Set;
 
 print "This is a test for: \n    \"This event happens from 13:00 to 14:00 every Tuesday, unless that Tuesday is the 15th of the month.\" (suggested by srl)\n\n";
-my $base_date = '20010501';
+my $base_date = '20010501Z';
 my $interval = Date::Set->new($base_date)->quantize(unit=>'months');
 # print "Weeks: ", $interval->quantize(unit=>'weeks'), "\n\n";
 my $tuesdays = $interval->quantize(unit=>'weeks')->
